#include "Pledge.h"
#include "memory/MemoryManager.h"
#include "sys/Syscall.h"
#include "thread/Scheduler.h"
#include <bits/pstat.h>

static void set_timespec(struct timespec& ts, u64 ticks)
{
    ts.tv_sec = ticks / 1000;
    ts.tv_nsec = (ticks % 1000) * 1000 * 1000;
}

Result<u64> sys_pstat(Registers*, SyscallArgs args)
{
    pid_t pid = (pid_t)args[0];
    struct process* ps = (struct process*)args[1];

    auto* current = Process::current();
    TRY(check_pledge(current, Promise::p_proc));

    // If pid == -1, return the PID of the last spawned thread.
    if (pid == -1) return g_processes.expect_last()->id;

    auto* target = TRY(Result<Process*>::from_option(Scheduler::find_by_pid(pid), ESRCH));

    struct process proc;
    proc.ps_pid = target->id;
    proc.ps_ppid = target->parent ? target->parent->id : 0;
    auto auth = target->credentials();
    proc.ps_uid = auth.uid;
    proc.ps_gid = auth.gid;
    proc.ps_euid = auth.euid;
    proc.ps_egid = auth.egid;
    proc.ps_state = 0; // FIXME: this is thread-specific now
    proc.ps_flags = 0; // FIXME: add flags
    set_timespec(proc.ps_time, target->user_ticks_self + target->kernel_ticks_self);
    set_timespec(proc.ps_ktime, target->kernel_ticks_self);
    set_timespec(proc.ps_utime, target->kernel_ticks_children);
    strlcpy(proc.ps_name, target->cmdline.chars(), sizeof(proc.ps_name));
    strlcpy(proc.ps_cwd, target->current_directory_path.is_empty() ? "/" : target->current_directory_path.chars(),
            sizeof(proc.ps_cwd));

    if (!MemoryManager::copy_to_user_typed(ps, &proc)) return err(EFAULT);

    return (u64)pid;
}
