#pragma once
#include "fs/VFS.h"
#include "thread/Thread.h"

namespace Scheduler
{
    void init();

    Thread* current();
    Thread* idle();
    Process* init_process();

    void set_reap_thread(Thread*);
    void signal_reap_thread();

    void set_oom_thread(Thread*);
    void signal_oom_thread();

    Result<Thread*> new_kernel_thread(u64 address, const char* name);
    Result<Thread*> new_kernel_thread(void (*func)(void), const char* name);
    Result<Thread*> new_kernel_thread(void (*func)(void*), void* arg, const char* name);

    Result<Thread*> create_init_process(SharedPtr<VFS::Inode> inode, const char* name);

    void add_thread(Thread* thread);
    void add_process(Process* process);

    Thread* pick_task();

    void reap_thread(Thread* thread);
    void reap_process(Process* thread);

    void switch_task(Registers* regs);

    void invoke(Registers* regs);

    LinkedList<Thread> check_for_dying_threads();
    LinkedList<Process> check_for_dead_processes();

    Option<Process*> find_by_pid(pid_t pid);
    Option<Thread*> find_by_tid(pid_t tid);

    template <typename Callback> void for_each_child(Process* process, Callback callback)
    {
        for (Process* current = process; current; current = g_processes.next(current).value_or(nullptr))
        {
            if (current->parent == process)
            {
                bool should_continue = callback(current);
                if (!should_continue) return;
            }
        }
    }

    template <typename Callback> void for_each_thread(Process* process, Callback callback)
    {
        for (Thread* current = g_threads.first().value_or(nullptr); current;
             current = g_threads.next(current).value_or(nullptr))
        {
            if (current->process == process)
            {
                bool should_continue = callback(current);
                if (!should_continue) return;
            }
        }
    }

    template <typename Callback> void for_each_in_process_group(pid_t group, Callback callback)
    {
        for (Process* current = g_processes.first().value_or(nullptr); current;
             current = g_processes.next(current).value_or(nullptr))
        {
            if (current->pgid == group)
            {
                bool should_continue = callback(current);
                if (!should_continue) return;
            }
        }
    }

    template <typename Callback> void for_each_in_session(pid_t sid, Callback callback)
    {
        for (Process* current = g_processes.first().value_or(nullptr); current;
             current = g_processes.next(current).value_or(nullptr))
        {
            if (current->sid == sid)
            {
                bool should_continue = callback(current);
                if (!should_continue) return;
            }
        }
    }

    void dump_state();

    bool has_children(Process* thread);

    Option<Process*> find_exited_child(Process* thread);
}

extern "C" void kernel_yield();
void kernel_wait(pid_t pid);
void kernel_sleep(u64 ms);
[[noreturn]] void kernel_exit();

// Freezes the current thread until someone else calls wake_up() on this thread.
void kernel_wait_for_event();
